@@ -16,6 +16,7 @@ class AstraEventHandler(AssistantEventHandler):
         self.tools = {}
         self.tool_output = ToolOutput
         self.tool_call_results = None
+        self.arguments = None
         self.stream = None
         self.error = None
 
@@ -30,9 +31,14 @@ class AstraEventHandler(AssistantEventHandler):
             self.tool_call_results = self.run_tool(tool_call)
         except Exception as e:
             self.error = e
+            raise e
 
         if not isinstance(self.tool_call_results, str) and self.tool_call_results is not None:
-            tool_call_results_string = self.tool_call_results["output"].to_string()
+            tool_call_results_obj = self.tool_call_results["output"]
+            if hasattr(tool_call_results_obj, 'to_string'):
+                tool_call_results_string = tool_call_results_obj.to_string()
+            else:
+                tool_call_results_string = tool_call_results_obj
         else:
             tool_call_results_string = self.tool_call_results
         self.tool_output = ToolOutput(
@@ -56,8 +62,10 @@ class AstraEventHandler(AssistantEventHandler):
                 arguments = json.loads(tool_call.function.arguments)
                 model: BaseModel = tool.get_model()
                 if issubclass(model, BaseModel):
-                    arguments = model(**arguments)
-                results = tool.call(arguments)
+                    self.arguments = model(**arguments)
+                else:
+                    self.arguments = arguments
+                results = tool.call(self.arguments)
                 return results
             except Exception as e:
                 self.logger.error(f"Error running tool {tool_name}: {e}")
